; BADVENTURE! 
; ATARI 2600 ADVENTURE REMAKE in BLITZ BASIC
; -because it hadn't been done yet-
;
; by morduun / morduun@hotmail.com
; Included code and media are not to be sold or included on any compilation disk
; without the express permissions of the original author, Warren Robinett, any other 
; existing copyright holders and myself, in that order.
;
; VERSION HISTORY
; ===============
; 2/14: miniproject started, map and object logic functional, bridge and magnet working
; 2/15: all maps connected, catacombs special rendering, gates and keys working
; 2/16: dragons and their AI in place; it's nearly a game now
; 2/17: bat in but buggy.  bat grabs objects, capturable.  minor overhaul to object ownership 
; 2/18: final dragon hunt mode fixed, few bugs squashed, game winnable.
; 2/19: chalice cycles colors, game winnable, reset button works.
; 2/20: minor tweaks and nudges, gold castle cycles colors when game won now.
; 2/21: two game modes: original and extra-crispy
; 2/23: bughunts continue. ;)
; 3/28: haven't touched it since 2/23, may as well release it ;)
;
; COMMENTS
; ========
; Originally a sort of last-minute 'I wonder if I could' for the 150-line contest,
; so there's lots of stuff here I don't normally use.  Objects, dragons and rooms
; would be far better defined as types instead of array entries, but as it was 
; originally meant for brevity of code (failed badly, unfortunately) I didn't do it 
; that way.
;
; KNOWN ISSUES
; ============
; As of 3/28, there are a few remaining bugs I have yet to chase down, mostly to do with
; trying to find the -1st element of an array.  Unfortunately they're difficult to duplicate,
; so if anyone can work out exactly where this happens and lets me know I'll fix the source.
;
; GAME HISTORY
; ============
; Adventure was originally released for the Atari 2600 in 1980 by Warren Robinett,
; and became famous for having the first 'Easter Egg' (which I have not yet duplicated
; in this release, but will eventually...  I think).  The entire program, complete
; with graphics and sound, fit in a 4k ROM chip with 128 bytes of RAM for variable
; storage.  This version uses screen grabs of all objects and rooms, and behaves as 
; close to the original as I can determine.  How Mr. Robinett managed all that in 4k of 
; code I'll never know but my hat's off to him.
;
; Other versions are available, including a Flash version that only replicates the
; easy level of the original, as well as 'Indenture,' a flawless DOS conversion that
; includes a new, much harder game mode, but which I can't find anymore on the net.
;
; OLDSCHOOLERS' NOTES
; ===================
; BADVENTURE does not include the classic 'Game 1' mode, where the catacombs, white 
; castle and black dungeon are omitted.  Technically, neither does it include an authentic 
; 'Game 2' mode where objects are placed in a specific preset pattern across the kingdom, 
; though the current Game 1 is similar in that object placement is consistent.  
; Game 2 is most similar to the classic 'Game 3' mode, where object, dragon and bat 
; placement is completely random, and, true to the spirit of the original, it can and will
; generate unwinnable maps!  Note also that, unlike the original, Yorgle the Yellow Dragon 
; is not afraid of the Yellow Key (and will guard it), all dragons hesitate for at least
; a moment before attempting to swallow the player, and there is no game mode where
; the dragons all run from the sword.  Also note the new behavior of the bridge: it will
; give you just enough rope to hang yourself with if you let it!
;
; CONTROLS
; ========
; GAME SELECT SCREEN
; Arrows to select game mode
; Space/Enter to start game
; [ESC] to quit
; GAME MODE 1: all items and creatures start in the same place, game always winnable
; GAME MODE 2: all items and creatures start randomized, game may not be winnable!
;
; GAME
; Up/Down/Left/Right for movement
; Spacebar to release an object you've picked up.
; [1] key to resurrect if you're eaten or trapped in a wall or oubliette
; [ESC] to go back to game select / quit screen


AppTitle "BADVENTURE!"
SeedRnd MilliSecs()
Graphics 320, 200, 32
SetBuffer BackBuffer()
ClsColor 186, 186, 186

; SCREEN = all adventure playfield images
; OBJ = all adventure play objects
; DRAGON = all dragon positions & states (0 = room, 1 = x, 2 = y, 3 = state, 4 = guarding, 5 = timer, 6 = flightpattern)
;			DRAGON #3 is the bat         (0 = room, 1 = x, 2 = y, 3 = flit, 4 = carrying, 5 = timer, 6 = flightpattern)
; OBJPOS = all object positions(0 = room, 1 = x, 2 = y)
; CASTLE = all castles' states (0 = iscastle, 1 = drawbrstate, 2 = direction, 3 = nextani, 4 = linkroom)
; RANDOBJ = spots for objects to be hidden (0 = room, 1 = x, 2 = y, 3 = object that CANNOT go here, 4 = suitable for chalice)

Dim Screen(30), Obj(22), Map(28, 4), DRAGON#(3, 6), objPos(22, 2), randObj(70, 4), castle%(28, 4), chalice(145, 1)
Global GameTime%, TargetFPS#=60.0, FrameDelay%=MilliSecs(), CurrentTicks%=MilliSecs(), SpeedFactor#, pX#=154, pY#=150, Room% = 1, Carry% = -1, PhaseWall% = False, LastMove% = -1, GateOpen%, GateClosed%, LastExit%, BatHappy%, BatBored%, nextChal = MilliSecs(), pMove% = True, VICTORY% = False, RANDSPOTS%
Global SFX_PICK = LoadSound("pickup.wav"), SFX_DROP = LoadSound("drop.wav"), cR=0, cG=0, cB=0, FLASHER% = CreateTimer(60)
Const t_objs=20, NORTH% = %1000, SOUTH% = %0010, WEST% = %0001, EAST% = 0100, ZONE% = 0, GUARD%=1, CHASE% = 2, HUNT% = 3, BITE% = 4, DEAD% = 5, ZONEBITE% = 6

; load up all the screens on the map
For i = 0 To 28
	Screen(i) = LoadImage(("screen" + i + ".png"))
	
	If screen(i) <> 0
		If ((i < 12) Or (i > 18)) 
			MaskImage screen(i), 186, 186, 186
		Else
			MaskImage screen(i), 255, 125, 0
		EndIf
	EndIf
	
	For j = 1 To 4
		Read map(i, j)
	Next
Next

; load up all the objects
For i = 0 To 19
	Obj(i) = LoadImage(("obj" + i + ".png"))
	If (obj(i) <> 0) 
		MaskImage obj(i), 186, 186, 186
	EndIf
Next


; load up chalice glow positions
Restore chalice
Read howMany
For i = 0 To howMany
	Read chalice(i, 0)
	Read chalice(i, 1)
Next

; load up all object random starting points
Restore randPlace
Read RANDSPOTS%
count = 0
For i = 0 To RANDSPOTS-1
	Read myRoom, myX, myY, myNo, myChalice, myEntries
	For j = 1 To myEntries
		randObj(count, 0) = myRoom
		randObj(count, 1) = myX
		randObj(count, 2) = myY
		randObj(count, 3) = myNo
		randObj(count, 4) = myChalice
		count = count + 1
	Next
Next

; load up castle gates
obj(20) = LoadAnimImage("obj20.png", 14, 32, 0, 8)
MaskImage obj(20), 186, 186, 186
obj(21) = LoadAnimImage("obj20.png", 14, 32, 0, 8)
MaskImage obj(21), 186, 186, 186
obj(22) = LoadAnimImage("obj20.png", 14, 32, 0, 8)
MaskImage obj(22), 186, 186, 186


; set castle gate positions appropriately
objpos(20, 0) = 1
objpos(21, 0) = 10
objpos(22, 0) = 24
objpos(20, 1) = 153
objpos(20, 2) = 117
objpos(21, 1) = 153
objpos(21, 2) = 115
objpos(22, 1) = 158
objpos(22, 2) = 117

; set castle phases and etc. appropriately
castle(1, 0) = True
castle(1, 3) = MilliSecs()
castle(1, 4) = 0

castle(10, 0) = True
castle(10, 3) = MilliSecs()
castle(10, 4) = 11

castle(24, 0) = True
castle(24, 3) = MilliSecs()
castle(24, 4) = 25

SetSpeedFactor()
SetSpeedFactor()

newGame()					; set up new game parameters

; main routine: rinse & repeat as necessary
Repeat 
	Cls
	GameTime = MilliSecs()
	SetSpeedFactor()
	MovePlayer()			; player input & collisions
	
	If Not VICTORY
		DoAnis()			; animate gates
		MoveDragons()		; move dragons around
		MoveBat()			; move bat around
		CheckWin()			; check to see if the player won
	EndIf
	
	CycleChalice()
	DrawScreen()
	Delay 10
	Flip
Forever

End

; for framerate independent movement
Function SetSpeedFactor()			; so we can still do this from within other functions
	CurrentTicks = GameTime
	SpeedFactor = (CurrentTicks - FrameDelay) / (1000 / TargetFPS)
	If SpeedFactor <= 0 Then SpeedFactor = 0.000000001
	FrameDelay = CurrentTicks
End Function

; this was going to be more multipurpose than this at one point, really.
; now it's really just to do castle gate animations
Function DoAnis()

	; gate animations
	For i = 0 To 28
		If castle(i, 0) = True	; if there's a castle here
			If (castle(i, 2) <> 0) ; if it's animating
				If castle(i, 3) < GameTime	; if it's time to animate
					castle(i, 1) = castle(i, 1) + castle(i, 2)
					If (castle(i, 1) = 0) Or (castle(i, 1) = 6)
						castle(i, 2) = 0
					Else
						castle(i, 3) = GameTime + 250
					EndIf
				EndIf
			EndIf
		EndIf
	Next
				

End Function


; check to see if the Chalice is either touching the gold gates
; or inside the gold vestibule
; if either is true, game is won!
Function CheckWin()

	; what room's the chalice in?
	chaliceRoom = objRoom(7)
	
	; if room > 1 we can't win
	If chaliceRoom > 1 Then Return
	
	; if in courtyard and gates are down, we can't win
	If (chaliceRoom = 1) And (castle(0, 1) = 0) Then Return
	
	; if in courtyard and chalice not touching gates, we can't win
	If ((chaliceRoom = 1) And (Not (ImagesCollide(obj(7), objX(7), objY(7), 0, obj(20), 153, 117, castle(0, 1))))) Then Return

	; if we've gotten this far, the player's won the game!

	For i = 1 To 8
		For j = 1 To 255
			If j <> 186
				newCol = (j*1000) + (j*100) + j
				For k = 1 To 255
					SetGamma k, k, k, (128 + ((k+j) Mod 128)), ((k-j) Mod 255), ((k+j) Mod 255)
				Next
				UpdateGamma
			EndIf
			start = MilliSecs() + 3
			While MilliSecs() < start
				
			Wend
		Next
	Next

	If DRAGON(0, 3) <> DEAD Then DRAGON(0, 3) = ZONE	; no AI
	If DRAGON(1, 3) <> DEAD Then DRAGON(0, 3) = ZONE	; no AI
	If DRAGON(2, 3) <> DEAD Then DRAGON(2, 3) = ZONE	; no AI
	DRAGON(3, 6) = 0									; no flight pattern

	For i = 0 To 255
		SetGamma i, i, i, i, i, i
	Next
	
	UpdateGamma

	VICTORY=1

End Function

; makes the chalice glow if it's in the same room as the player
Function CycleChalice()

	If ObjRoom(7) <> Room Then Return
	If nextChal > GameTime Then Return
	
	nextChal = GameTime + 10

	cR = cR + 1
	
	If cR > 255 Then cR = 0
	
	newCol = (cR*1000) + (cR*100) + cR
	
	SetBuffer ImageBuffer(obj(7))
	LockBuffer ImageBuffer(obj(7))
	
	For i = 0 To 145
		WritePixel chalice(i, 0), chalice(i, 1), newCol
	Next
	UnlockBuffer ImageBuffer(obj(7))
	SetBuffer BackBuffer()
		

End Function

; for when the player gets eaten or stuck in a wall
; drops all objects, brings player back to gold castle
; but also resurrects all dragons!
Function Resurrect()
	
	DropObj()	; no, you can't take it with you
	pX#=154		; reset x pos
	pY#=150		; reset y pos
	Room = 1	; reset to gold castle
	
	; Resurrect the dragons too!  w00t!
	If DRAGON(0, 3) = DEAD Then DRAGON(0, 3) = HUNT
	If DRAGON(1, 3) = DEAD Then DRAGON(1, 3) = HUNT
	If DRAGON(2, 3) = DEAD Then DRAGON(2, 3) = HUNT

End Function


; yeah, so I cheated.
; this really shouldn't be a separate game loop, but I just didn't want to bother
; with incorporating it into the main loop.
; Game Mode 1 replaces all objects, Dragons and Bat in their default positions
; Game Mode 2 sets all objects, Dragons and Bat in (almost) totally random positions
Function newGame()

	While KeyDown(1)
		Delay 10
	Wend
	FlushKeys()

	pX#=154
	pY#=150
	Room% = 1
	Carry% = -1
	PhaseWall% = False
	LastMove% = -1
	nextChal = MilliSecs()
	pMove% = True
	VICTORY% = False
	
	Local GameMode = False
	Local WhichMode = 1
	Color 0, 128, 0

	While Not GameMode
		Cls

		DrawBlock screen(22), 0, 4
		Text 160, 100, WhichMode, True, True
		
		Delay 10
		
		If KeyHit(1)
			End
		ElseIf KeyHit(2)
			WhichMode = 1
		ElseIf KeyHit(3)
			WhichMode = 2
		ElseIf (KeyHit(200)) Or (KeyHit(203))
			WhichMode = WhichMode - 1
			If WhichMode < 1 Then WhichMode = 2
		ElseIf (KeyHit(205)) Or (KeyHit(208))
			WhichMode = WhichMode + 1
			If WhichMode > 2 Then WhichMode = 1
		ElseIf (KeyHit(28)) Or (KeyHit(57))
			GameMode = WhichMode
		EndIf
					
		Flip
	Wend
	
	Select GameMode
	
		Case 1	; everything starts the same
		
			DRAGON(0, 0) = 3		; yorgle in gold crossroads west
			DRAGON(0, 4) = 4		; guarding the gold key

			DRAGON(1, 0) = 28		; grundle in the red labyrinth
			DRAGON(1, 4) = 5		; guarding the black key

			DRAGON(2, 0) = 15		; rhindle in the black dungeon
			DRAGON(2, 4) = 7		; guarding the chalice
			
			DRAGON(3, 0) = 0		; bat in the gold castle vestibule
			
			Restore items
			For i = 0 To 8
				Read objpos(i, 0)	; map
				Read objpos(i, 1)	; x
				Read objpos(i, 2)	; y
			Next

		Case 2	; everything starts randomized
			Where = Rand(0, RANDSPOTS-1)
			DRAGON(0, 0) = RandObj(Where, 0)
			DRAGON(0, 1) = RandObj(Where, 1)
			DRAGON(0, 2) = RandObj(Where, 2)

			Where = Rand(0, RANDSPOTS-1)
			DRAGON(1, 0) = RandObj(Where, 0)
			DRAGON(1, 1) = RandObj(Where, 1)
			DRAGON(1, 2) = RandObj(Where, 2)

			Where = Rand(0, RANDSPOTS-1)
			DRAGON(2, 0) = RandObj(Where, 0)
			DRAGON(2, 1) = RandObj(Where, 1)
			DRAGON(2, 2) = RandObj(Where, 2)
			
			Where = Rand(0, RANDSPOTS-1)
			DRAGON(3, 0) = RandObj(Where, 0)
			DRAGON(3, 1) = RandObj(Where, 1)
			DRAGON(3, 2) = RandObj(Where, 2)
			
			For i = 0 To 8
				Where = Rand(0, RANDSPOTS-1)
				If i <> 7
					While RandObj(Where, 3) = i
						Where = Rand(0, RANDSPOTS-1)
					Wend
				Else
					While RandObj(Where, 4) = False
						Where = Rand(0, RANDSPOTS-1)
					Wend
				EndIf
				
				objPos(i, 0) = RandObj(Where, 0)
				objPos(i, 1) = RandObj(Where, 1)
				objPos(i, 2) = RandObj(Where, 2)
			Next

		
	End Select
	
	DRAGON(0, 3) = HUNT
	DRAGON(1, 3) = HUNT
	DRAGON(2, 3) = HUNT
	
	DRAGON(3, 5) = MilliSecs()
	
	castle(1, 1) = 0
	castle(10, 1) = 0
	castle(24, 1) = 0

End Function


; AI behavior here
; bat will become gradually less and less happy with whatever it's carrying.
; bat will randomly fly in straight lines (horiz, vert or diag) for periods of time
;  until it decides it wants something else; then it beelines for a new object and 
;  swaps it for whatever it currently has
; player can catch the bat, but the bat's tricksy, and will eventually escape
;  if the player holds on to it for too long.
Function MoveBat()

	If GameTime > DRAGON(3, 5)
		DRAGON(3, 5) = GAMETIME + 250
		DRAGON(3, 3) = Not DRAGON(3, 3)
	EndIf
	
	If DRAGON(3, 3) = False
		myBat = 9
	Else
		myBat = 10
	EndIf
	
	bspeed# = 1.75 * SpeedFactor
	
	; if the bat isn't carrying anything, or if he's tired of his current object
	If DRAGON(3, 4) = 0 Or (GameTime > BatHappy) Or ((BatBored = True) And (GameTime < BatHappy))
		For i = 1 To 7									; are there any items around to pick up?
			If objPos(i, 0) = DRAGON(3, 0)
				If DRAGON(3, 1) < objpos(i, 1)
					DRAGON(3, 1) = DRAGON(3, 1) + bspeed
				Else
					DRAGON(3, 1) = DRAGON(3, 1) - bspeed
				EndIf

				If DRAGON(3, 2) < objpos(i, 2)
					DRAGON(3, 2) = DRAGON(3, 2) + bspeed
				Else
					DRAGON(3, 2) = DRAGON(3, 2) - bspeed
				EndIf
				
				If ImagesCollide(obj(myBat), DRAGON(3, 1), DRAGON(3, 2), 0, obj(i), objpos(i, 1), objpos(i, 2), 0)
					BatDropObj()
					BatGetObj(i)
					DRAGON(3, 6) = Rand(1, 8)	; new flight pattern
				EndIf
								
				Return
				
			ElseIf (objPos(i, 0) = -1) And (DRAGON(3, 0) = Room)	; ooo, player has something cool!
			
				oPx# = ObjX(i)
				oPy# = objY(i)
				
				If DRAGON(3, 1) < oPx
					DRAGON(3, 1) = DRAGON(3, 1) + bspeed
				Else
					DRAGON(3, 1) = DRAGON(3, 1) - bspeed
				EndIf

				If DRAGON(3, 2) < oPy
					DRAGON(3, 2) = DRAGON(3, 2) + bspeed
				Else
					DRAGON(3, 2) = DRAGON(3, 2) - bspeed
				EndIf
				
				If ImagesCollide(obj(myBat), DRAGON(3, 1), DRAGON(3, 2), 0, obj(i), objX(i), objY(i), 0)
					DropObj()					; force the player to drop the object!
					BatDropObj()				; bat drops whatever he was carrying
					BatGetObj(i)				; bat gets the new object
					DRAGON(3, 6) = Rand(1, 8)	; new flight pattern
				EndIf
								
				Return
			
			EndIf
			
		Next
		
		For i = 0 To 2
			If DRAGON(i, 0) = DRAGON(3, 0)				; if not, we can always cart around dragons
			
				If DRAGON(3, 1) < DRAGON(i, 1)
					DRAGON(3, 1) = DRAGON(3, 1) + bspeed
				Else
					DRAGON(3, 1) = DRAGON(3, 1) - bspeed
				EndIf

				If DRAGON(3, 2) < DRAGON(i, 2)
					DRAGON(3, 2) = DRAGON(3, 2) + bspeed
				Else
					DRAGON(3, 2) = DRAGON(3, 2) - bspeed
				EndIf
				
				If ImagesCollide(obj(myBat), DRAGON(3, 1), DRAGON(3, 2), 0, obj(11), DRAGON(i, 1), DRAGON(i, 2), 0)
					BatDropObj()
					BatGetDragon(i)
				EndIf
								
				Return			
			
			EndIf
		Next

		; last resort; change the flight pattern (also attempt escape here)
		If (GameTime > BatHappy)
			BatHappy = GameTime + Rand(5000, 20000)
			DRAGON(3, 6) = Rand(1, 8)
			BatBored% = True
			If DRAGON(3, 0) = -1 Then DropObj()		; attempt to escape the player!
		EndIf
	
	EndIf
	
		
	If DRAGON(3, 0) <> -1					; if the player doesn't have us, just keep flittin.

		Select DRAGON(3, 6)						; select flight pattern
		
			Case 1	; goin up
				DRAGON(3, 2) = DRAGON(3, 2) - bspeed
				
			Case 2	; goin east
				DRAGON(3, 1) = DRAGON(3, 1) + bspeed
				
			Case 3	; goin south
				DRAGON(3, 2) = DRAGON(3, 2) + bspeed
				
			Case 4	; goin west
				DRAGON(3, 1) = DRAGON(3, 1) - bspeed
				
			Case 5	; goin ne
				DRAGON(3, 1) = DRAGON(3, 1) + bspeed
				DRAGON(3, 2) = DRAGON(3, 2) - bspeed
				
			Case 6	; goin se
				DRAGON(3, 1) = DRAGON(3, 1) + bspeed
				DRAGON(3, 2) = DRAGON(3, 2) + bspeed
				
			Case 7	; goin sw
				DRAGON(3, 1) = DRAGON(3, 1) - bspeed
				DRAGON(3, 2) = DRAGON(3, 2) + bspeed
				
			Case 8	; goin nw
				DRAGON(3, 1) = DRAGON(3, 1) - bspeed
				DRAGON(3, 2) = DRAGON(3, 2) - bspeed
		
		
		End Select
		
		If DRAGON(3, 2) < -16		; leaving north
			DRAGON(3, 0) = map(DRAGON(3, 0), 1)
			DRAGON(3, 2) = 216
			
		ElseIf DRAGON(3, 2) > 216	; leaving south: check for locked castle doors here
			; if trying to leave a castle but the gate's locked, wrap the bugger
			If ((castle(map(DRAGON(3, 0), 3), 0) = True) And (castle(map(DRAGON(3, 0), 3), 1) = False))
				DRAGON(3, 2) = -16
			
			; if trying to leave a castle and the gate's open, put him at the entrance
			ElseIf ((castle(map(DRAGON(3, 0), 3), 0) = True) And (castle(map(DRAGON(3, 0), 3), 1) <> False))
				DRAGON(3, 0) = map(DRAGON(3, 0), 3)
				DRAGON(3, 2) = 140
				DRAGON(3, 1) = 156
			
			; otherwise, carry on.
			Else
				DRAGON(3, 0) = map(DRAGON(3, 0), 3)
				DRAGON(3, 2) = -16
			EndIf
			
		ElseIf DRAGON(3, 1) < -16	; leaving west
			DRAGON(3, 0) = map(DRAGON(3, 0), 4)
			DRAGON(3, 1) = 336
			
		ElseIf DRAGON(3, 1) > 336	; leaving east
			DRAGON(3, 0) = map(DRAGON(3, 0), 2)
			DRAGON(3, 1) = -16
			
		EndIf
		
	
	EndIf
	
	

End Function


; AI behavior here
; by default, dragons HUNT: roaming the playfield for either the player or a favored object
; while HUNTING, if they come across the player, they will give CHASE
; while HUNTING, if they come across a favored object, they will GUARD it
; while GUARDING, if the player enters their room, they will pursue but not past the screen.
; while GUARDING, if the player steals their object they will CHASE
; while CHASING, if they come across a favored object they will GUARD it
; if a dragon collides with a player it will BITE
; if the player is still colliding with a dragon when it's done BITING, player dies
; if the bat picks up a dragon, it will ZONE until dropped, when it will HUNT again
; if a dragon collides with the sword, it will be DEAD.

; so:  HUNT, GUARD, CHASE, BITE, ZONE & DEAD are possible states.

Function MoveDragons()

	For i = 0 To 2
		If DRAGON(i, 0) <> -2
			dspeed# = (Float(i * 0.25) + 1.5) * SpeedFactor
			Select DRAGON(i, 3)
			
				Case ZONE	; do nuttin, but dragon still alive.
			
				Case GUARD	; guarding an object
					
					If objpos(DRAGON(i, 4), 0) = -1	; if the player grabbed our guard object
						DRAGON(i, 3) = CHASE		; chase the bastard!
						DRAGON(i, 4) = 0
						
					ElseIf objpos(DRAGON(i, 4), 0) = -2	; if the bat grabbed our guard object
						DRAGON(i, 3) = HUNT			; find something else to do
						DRAGON(i, 5) = Rand(5000, 30000)
						DRAGON(i, 6) = Rand(1, 8)
	
					ElseIf dragon(i, 0) = Room		; if the player's in the room
						DragonToPlayer(i, False)	; chase him out
	
					Else 							; otherwise, be sure we're on top of our object
						If DRAGON(i, 1) < objpos(DRAGON(i, 4), 1)
							DRAGON(i, 1) = DRAGON(i, 1) + dspeed
						ElseIf DRAGON(i, 1) > objpos(DRAGON(i, 4), 1)
							DRAGON(i, 1) = DRAGON(i, 1) - dspeed
						EndIf
						
						If DRAGON(i, 2) > objpos(DRAGON(i, 4), 2) 
							DRAGON(i, 2) = DRAGON(i, 2) - (dspeed)
						ElseIf DRAGON(i, 2) < objpos(DRAGON(i, 4), 2) 
							DRAGON(i, 2) = DRAGON(i, 2) + (dspeed)
						EndIf					
	
					EndIf
					
					myDrag = (i * 3) + 11
				
				Case CHASE	; chasing the player
					DRAGON(i, 4) = DragonGuard(i)	; see if there's anything in the room worth guarding
					If DRAGON(i, 4) = False			; if not, keep chasin!
						DragonToPlayer(i, True)
					Else							; otherwise, set up camp
						DRAGON(i, 3) = GUARD
					EndIf
					
					myDrag = (i * 3) + 11
				
				Case HUNT	; hunting something; either the player or something to guard

					DragonHunt(i)

					myDrag = (i * 3) + 11
					
				Case BITE, ZONEBITE	; getting ready to swallow the player
					If DRAGON(i, 5) < GameTime 
						If RectsOverlap(px, py, 8, 8, DRAGON(i, 1)-1, DRAGON(i, 2), 11, 20)	; bad news!
							DRAGON(i, 3) = 0	; "sit still" mode
							px = DRAGON(i, 1) + 5
							py = DRAGON(i, 2) + 20
						Else
							If DRAGON(i, 3) = ZONEBITE
								DRAGON(i, 3) = ZONE
							Else
								DRAGON(i, 4) = DragonGuard(i)
								If DRAGON(i, 4) = False
									DRAGON(i, 3) = CHASE
									DragonToPlayer(i, True)
								Else
									DRAGON(i, 3) = GUARD
								EndIf
							EndIf
						EndIf
					EndIf
					
					myDrag = (i * 3) + 12
					
				Case DEAD	; heh!
									
				
			End Select
		EndIf
		
		DragonDeath(i)
		
		DragonBite(i)
		
	Next

End Function

; check to see if a dragon is (A) not dead, and (B) colliding with the sword
; if so and if sword and dragon collide, kill the dragon!
Function DragonDeath(drag)
	If DRAGON(drag, 3) <> DEAD
		If (DragRoom(drag) = Room) And (ObjRoom(1) = Room)
			If ImagesCollide(obj(11), DragX(drag), DragY(drag), 0, obj(1), objX(1), objY(1), 0)
				DRAGON(drag, 3) = DEAD
			EndIf
		EndIf
	EndIf
End Function

; check to see if a dragon is (A) not dead, (B) not already biting, and (C) in the player room
; if so and if player and dragon collide, bite!
Function DragonBite(drag)

	If DragRoom(drag) <> Room Then Return
	If DRAGON(drag, 3) = DEAD Then Return
	If DRAGON(drag, 3) = BITE Then Return
	If DRAGON(drag, 3) = ZONEBITE Then Return
	
	If ImagesCollide(obj(11), DragX(drag), DragY(drag), 0, obj(0), px, py, 0)

		Local speed# = Float(drag * 0.25) + 1.5
	
		If DRAGON(drag, 3) = ZONE
			DRAGON(drag, 3) = ZONEBITE
		Else
			DRAGON(drag, 3) = BITE
		EndIf
				
		DRAGON(drag, 5) = ((3.0 - speed) * 750) + GameTime
		DRAGON(drag, 1) = pX + 8
		DRAGON(drag, 2) = pY
	EndIf

End Function

; moves the specified dragon towards the player
; if leaveroom = false, the dragon will not leave his room to give chase
; if leaveroom = true, dragon will transition screens to chase player
Function DragonToPlayer(drag, leaveroom=False)

	Local speed# = (Float(drag * 0.25) + 1.5) * SpeedFactor

	oldX# = DRAGON(drag, 1)
	oldY# = DRAGON(drag, 2)
	
	If DragRoom(drag) = Room
	
		If DRAGON(drag, 1) < pX
			DRAGON(drag, 1) = DRAGON(drag, 1) + speed
		Else
			DRAGON(drag, 1) = DRAGON(drag, 1) - speed
		EndIf
		
		If DRAGON(drag, 2) < pY
			DRAGON(drag, 2) = DRAGON(drag, 2) + speed 
		Else
			DRAGON(drag, 2) = DRAGON(drag, 2) - speed 
		EndIf
		
	Else
	
		If leaveroom = True
			
			Select LastExit%
				
				Case NORTH
					myNewX# = DRAGON(drag, 1)
					myNewY# = 200.0 + DRAGON(drag, 2)
				
				Case SOUTH
					If Castle(Room, 0) = True And Castle(Room, 3) = 0	; if the gate is shut, we're trapped!
						DRAGON(drag, 3) = HUNT
						Return
					Else
						myNewX# = DRAGON(drag, 1)
						myNewY# = DRAGON(drag, 2) - 196
					EndIf
					
				Case EAST
					myNewX# = DRAGON(drag, 1) - 316
					myNewY# = DRAGON(drag, 2)
				
				Case WEST
					myNewX# = 320 + DRAGON(drag, 1)
					myNewY# = DRAGON(drag, 2)
				
			End Select

			DRAGON(drag, 0) = Room
			DRAGON(drag, 1) = myNewX
			DRAGON(drag, 2) = myNewY
		
		EndIf
	
	EndIf
		
End Function


; checks to see if the dragon's in the same room as something it'd like to guard
; if not, returns false
; if so, returns the object number
Function DragonGuard(drag)

	; note that nobody guards the sword!  this is so they don't end up killing themselves
	; in the process of guarding the damn thing ;)
	Select drag
		
		Case 0	; yorgle will guard the chalice, the gold key or the magnet
			If DRAGON(drag, 0) = objpos(7, 0)	; if he's in the same room as the chalice
				Return 7
			ElseIf DRAGON(drag, 0) = objpos(4, 0);	same room as the gold key
				Return 4
			ElseIf DRAGON(drag, 0) = objpos(2, 0);	same room as the magnet
				Return 3
			Else
				Return False
			EndIf
			
		
		Case 1	; grundle will guard the bridge or the black key 
			If DRAGON(drag, 0) = objpos(3, 0)	; if he's in the same room as the bridge
				Return 3
			ElseIf DRAGON(drag, 0) = objpos(5, 0);	same room as the black key
				Return 5
			Else
				Return False
			EndIf
					
		Case 2	; rhindle will guard the chalice or the white key
			If DRAGON(drag, 0) = objpos(7, 0)	; if he's in the same room as the chalice
				Return 7
			ElseIf DRAGON(drag, 0) = objpos(6, 0);	same room as the white key
				Return 6
			Else
				Return False
			EndIf
		
	End Select
	
End Function

; this moves the dragon from room to room when we're bored and looking for something to do.
; also changes state appropriately when the dragon spots something better to do 
; (ie GUARD or CHASE)
Function DragonHunt(drag)

	Local speed# = (Float(drag * 0.25) + 1.5) * SpeedFactor

	If DRAGON(drag, 0) = Room			; if we're in the same room as the player
		DRAGON(drag, 3) = CHASE
		Return
	EndIf
		
	DRAGON(drag, 4) = DragonGuard(i)	; if there's something here worth guarding
	If DRAGON(drag, 4) <> False
		DRAGON(drag, 3) = GUARD
		Return
	EndIf
	
	If DRAGON(drag, 5) < GameTime	; if we're bored, change our flight pattern
		DRAGON(drag, 6) = Rand(1, 8)
		DRAGON(drag, 5) = GameTime + Rand(5000, 30000)
	EndIf
	
	Select DRAGON(drag, 6)			; select flight pattern
		
		Case 1	; goin up
			DRAGON(drag, 2) = DRAGON(drag, 2) - speed
			
		Case 2	; goin east
			DRAGON(drag, 1) = DRAGON(drag, 1) + speed
			
		Case 3	; goin south
			DRAGON(drag, 2) = DRAGON(drag, 2) + speed
			
		Case 4	; goin west
			DRAGON(drag, 1) = DRAGON(drag, 1) - speed
			
		Case 5	; goin ne
			DRAGON(drag, 1) = DRAGON(drag, 1) + speed
			DRAGON(drag, 2) = DRAGON(drag, 2) - speed
			
		Case 6	; goin se
			DRAGON(drag, 1) = DRAGON(drag, 1) + speed
			DRAGON(drag, 2) = DRAGON(drag, 2) + speed
			
		Case 7	; goin sw
			DRAGON(drag, 1) = DRAGON(drag, 1) - speed
			DRAGON(drag, 2) = DRAGON(drag, 2) + speed
			
		Case 8	; goin nw
			DRAGON(drag, 1) = DRAGON(drag, 1) - speed
			DRAGON(drag, 2) = DRAGON(drag, 2) - speed
	
	
	End Select
		
	If DRAGON(drag, 2) < -16		; leaving north
		DRAGON(drag, 0) = map(DRAGON(drag, 0), 1)
		DRAGON(drag, 2) = 216
		
	ElseIf DRAGON(drag, 2) > 216	; leaving south: check for locked castle doors here
		; if trying to leave a castle but the gate's locked, wrap the bugger
		If ((castle(map(DRAGON(drag, 0), 3), 0) = True) And (castle(map(DRAGON(drag, 0), 3), 1) = False))
			DRAGON(drag, 2) = -16
		
		; if trying to leave a castle and the gate's open, put him at the entrance
		ElseIf ((castle(map(DRAGON(3, 0), 3), 0) = True) And (castle(map(DRAGON(drag, 0), 3), 1) <> False))
			DRAGON(drag, 0) = map(DRAGON(drag, 0), 3)
			DRAGON(drag, 2) = 140
			DRAGON(drag, 1) = 156
		
		; otherwise, carry on.
		Else
			DRAGON(drag, 0) = map(DRAGON(drag, 0), 3)
			DRAGON(drag, 2) = -16
		EndIf
		
	ElseIf DRAGON(drag, 1) < -16	; leaving west
		DRAGON(drag, 0) = map(DRAGON(drag, 0), 4)
		DRAGON(drag, 1) = 336
		
	ElseIf DRAGON(drag, 1) > 336	; leaving east
		DRAGON(drag, 0) = map(DRAGON(drag, 0), 2)
		DRAGON(drag, 1) = -16
		
	EndIf
		
End Function


; I never meant for this routine to be so messy; originally it was just player movement
; but it became simpler to run collision detection and screen transition here as well
; since all player movement went through this routine anyway
; and THEN it was easy to add the magnetism routine here since I was already looping
; through all the objects for something else.  so, unfortunately, this routine is a 
; victim of some nasty code creep.  Ah well.
Function MovePlayer()
	firstPX# = px
	firstPY# = py
	
	; esc key to go back to screen select
	If KeyDown(1)
		newGame()
		Return
	EndIf
	
	; '1' key to resurrect
	If KeyDown(2)
		Resurrect()
		Return
	EndIf
	
	; if we've won, those are our only options
	If VICTORY = True
		Return
	EndIf

	; up key for up
	If KeyDown(200) 
		pY = pY - (2.0 * SpeedFactor)
		LastMove = NORTH
	EndIf
	
	; down key for down
	If KeyDown(208) 	
		pY = pY + (2.0 * SpeedFactor)
		LastMove = SOUTH
	EndIf
	
	; if the bridge isn't there and we're hitting the room, move us back.
	If (Not PhaseWall) And (ImagesCollide(obj(0), px, py, 0, screen(Room), 0, 4, 0))
		px = firstPX
		py = firstPY
	EndIf
	
	oldPX# = px
	oldPY# = py
	
	; left key for left
	If KeyDown(203) 
		pX = pX - (2.0 * SpeedFactor)
		LastMove = LastMove Or WEST
	EndIf
		
	; right key for right
	If KeyDown(205) 
		pX = pX + (2.0 * SpeedFactor)
		LastMove = LastMove Or EAST
	EndIf

	; can't use the bridge for l/r movement, so
	; if we're hitting the room, move us back
;	If (Not PhaseWall) And (ImagesCollide(obj(0), px, py, 0, screen(Room), 0, 4, 0))
	If ImagesCollide(obj(0), px, py, 0, screen(Room), 0, 4, 0)
		px = oldPX
		py = oldPY
	EndIf

	; spacebar to drop our carried object
	If KeyHit(57) Then DropObj()
	
	; if we're below the playfield, move us to the room to the south (via gate if necessary)
	If py > 196
		If (objpos(3, 0) = Room) And (objpos(3, 2) > 143)
			objpos(3, 0) = map(room, 3)
			objpos(3, 2) = 199 - 56 -  objpos(3, 2)
		EndIf
		oldRoom = room
		room = map(room, 3)
		px = firstPX
		If (castle(room, 0) = True) And oldRoom = castle(room, 4)
			px = 156
			py = 140
		Else
			py = 6
		EndIf
		
		LastExit = SOUTH

	; if we're above the playfield, move us to the room to the north
	ElseIf py < 4
		If (objpos(3, 0) = Room) And (objpos(3, 2) < 0)
			objpos(3, 0) = map(room, 1)
			objpos(3, 2) = 199 + objpos(3, 2)
		EndIf
		
		px = firstPX
		room = map(room, 1)
		py = 194
		LastExit = NORTH
	
	; if we're right of the playfield, move us to the room to the east.
	ElseIf px > 318
		room = map(room, 2)
		px = 6
		py = firstPY
		LastExit = EAST
	
	; if we're left of the playfield, move us to the room to the west
	ElseIf px < 4
		room = map(room, 4)
		px = 316
		py = firstPY
		LastExit = WEST
		
	EndIf
	
	; check all objects
	For i = 1 To 22
		If objpos(i, 0) = Room	; if the object's in the same room as the player
			If i < 8	; takeable objects; should be 8 (9 w/microdot)
				; and if the player's colliding with the object
				If ImagesCollide(obj(0), px, py, 0, obj(i), objpos(i, 1), objpos(i, 2), 0)
					DropObj()	; drop whatever we're carrying
					GetObj(i)	; and pick up the new object
				EndIf
				
			ElseIf i > 19	; checking the castle gates
				; if the player and a set of gates are colliding
				If ImagesCollide(obj(0), px, py, 0, obj(i), objpos(i, 1), objpos(i, 2), castle(room, 1))
					If castle(room, 1) > 0	; if the portcullis is open
						room = castle(room, 4)	; move the player into the gateroom
						py = 194
					Else					; otherwise, bump 'em back
						px = FirstPX
						py = FirstPY
					EndIf	
				EndIf
					
				Select i	; check each gate to see if they're touching their key
					Case 20	; gold gate
						If (objpos(4, 0) = -1) Or (objpos(4, 0) = room)	; if the gold key's in the room
						
							; get it's current and accurate position
							If objPos(4, 0) = -1
								keyX# = objPos(4, 1) + pX
								keyY# = objPos(4, 2) + pY
							Else
								keyX# = objPos(4, 1)
								keyY# = objPos(4, 2)
							EndIf
							
							If ImagesCollide(obj(4), keyX#, keyY#, 0, obj(i), objpos(i, 1), objpos(i, 2), castle(room, 1))
								If castle(room, 1) = 6
									castle(room, 2) = -1
								ElseIf castle(room, 1) = 0
									castle(room, 2) = 1
								EndIf
							EndIf
						EndIf									
						
						
					Case 21	; black gate
						If (objpos(5, 0) = -1) Or (objpos(5, 0) = room)	; if the black key's in the room
						
							; get it's current and accurate position
							If objPos(5, 0) = -1
								keyX# = objPos(5, 1) + pX
								keyY# = objPos(5, 2) + pY
							Else
								keyX# = objPos(5, 1)
								keyY# = objPos(5, 2)
							EndIf
							
							If ImagesCollide(obj(5), keyX#, keyY#, 0, obj(i), objpos(i, 1), objpos(i, 2), castle(room, 1))
								If castle(room, 1) = 6
									castle(room, 2) = -1
								ElseIf castle(room, 1) = 0
									castle(room, 2) = 1
								EndIf
							EndIf
						EndIf					

					Case 22	; white gate
						If (objpos(6, 0) = -1) Or (objpos(6, 0) = room)	; if the gold key's in the room
						
							; get it's current and accurate position
							If objPos(6, 0) = -1
								keyX# = objPos(6, 1) + pX
								keyY# = objPos(6, 2) + pY
							Else
								keyX# = objPos(6, 1)
								keyY# = objPos(6, 2)
							EndIf
							
							If ImagesCollide(obj(6), keyX#, keyY#, 0, obj(i), objpos(i, 1), objpos(i, 2), castle(room, 1))
								If castle(room, 1) = 6
									castle(room, 2) = -1
								ElseIf castle(room, 1) = 0
									castle(room, 2) = 1
								EndIf
							EndIf
						EndIf
						
				End Select

			
			EndIf
		EndIf
		
		; check all small "magnetic" objects if they're in the same room as the magnet
		If (i < 8) And (i <> 2) And (ObjRoom(2) = ObjRoom(i)) And (objPos(i, 0) > -1)

			magX# = ObjX(2)
			magY# = ObjY(2)
			
			oX# = ObjX(i)
			oY# = ObjY(i)

			If magX - oX > 1
				 objpos(i, 1) = objpos(i, 1) + (0.5 * SpeedFactor)
			ElseIf magX - oX < -1
				objpos(i, 1) = objpos(i, 1) - (0.5 * SpeedFactor)
			EndIf

			If magY - oY > -15
				 objpos(i, 2) = objpos(i, 2) + (0.5 * SpeedFactor)
			ElseIf magY - oY < -16
				objpos(i, 2) = objpos(i, 2) - (0.5 * SpeedFactor)
			EndIf
		EndIf
			

	Next

; bridge phasing	
	If (objpos(3, 0) = Room) And (RectsOverlap(px, py, 8, 8, objpos(3, 1), objpos(3, 2), 56, 56))
		PhaseWall = True
	Else
		PhaseWall = False
	EndIf
	
	For i = 0 To 2		; dragon collisions
		If DRAGON(i, 0) = Room
			
			Select DRAGON(i, 3)
			
				Case DEAD
					myDrag = (i * 3) + 13
				
				Case BITE
					myDrag = (i * 3) + 12
				
				Default
					myDrag = (i * 3) + 11
			End Select
			
			If ImagesCollide(obj(0), px, py, 0, obj(myDrag), DRAGON(i, 1), DRAGON(i, 2), 0)
				pX = firstPX
				pY = firstPY
				Exit
			EndIf
		
		EndIf
	Next
	
	If DRAGON(3, 0) = Room		; bat collisions
	
		If DRAGON(3, 3) = False
			myBat = 9
		Else
			myBat = 10
		EndIf
		
		If ImagesCollide(obj(0), pX, pY, 0, obj(myBat), DRAGON(3, 1), DRAGON(3, 2), 0)
			DropObj()
			GetBat()
		EndIf
		
	EndIf
	
	If Not pMove 
		pX = firstX
		pY = firstY
	EndIf
	
End Function

; draws all screens and objects
Function DrawScreen()
	
	; draw the room
	; ==================================================
	If (room < 12) Or (room > 18)
		DrawBlock screen(Room), 0, 4
	Else
		DrawBlockRect screen(Room), px-24, py-24, px-24, py-28, 52, 52
	EndIf
	
	; draw the player
	; ==================================================
	DrawBlock obj(0), px, py
	
	; draw the objects & gates
	; ==================================================
	For i = 1 To 22
		If objRoom(i) = Room
			If i < 8
				DrawImage obj(i), objX(i), objY(i)
			ElseIf i > 19
				DrawImage obj(i), objpos(i, 1), objpos(i, 2), castle(room, 1)
			EndIf
			
		EndIf
	Next
	
	; draw the dragons
	; ==================================================
	For i = 0 To 2 
		If DragRoom(i) = Room
			Select DRAGON(i, 3)
		
				Case BITE
					myDrag = 12 + (3 * i)
					
				Case DEAD
					myDrag = 13 + (3 * i)
			
				Default
					myDrag = 11 + (3 * i)
			End Select
			
			DrawImage obj(myDrag), DragX(i), DragY(i)

		EndIf
	Next
	
	
	; draw the bat
	; ==================================================
	If (DRAGON(3, 0) = Room) Or (DRAGON(3, 0) = -1)
	
		If DRAGON(3, 3) = False
			myBat = 9
		Else
			myBat = 10
		EndIf
		
		If Dragon(3, 0) = -1
			myX# = pX + DRAGON(3, 1)
			myY# = pY + DRAGON(3, 2)
		Else
			myX# = DRAGON(3, 1)
			myY# = DRAGON(3, 2)
		EndIf
		
		DrawImage obj(myBat), myX, myY
	
	EndIf
End Function

; tells what room a dragon's in, despite ownership
Function DragRoom(myDrag)
	If DRAGON(myDrag, 0) > -1 Then Return DRAGON(myDrag, 0)
	If ((DRAGON(myDrag, 0) = -2) And (DRAGON(3, 0) <> -1)) Then Return DRAGON(3, 0)
	Return Room
End Function

; returns X-coord of dragon, despite ownership
Function DragX#(myDrag)
	If DRAGON(myDrag, 0) > -1 Then Return DRAGON(myDrag, 1)
	If ((DRAGON(myDrag, 0) = -2) And (DRAGON(3, 0) <> -1)) Then Return Float(DRAGON(3, 1) + DRAGON(myDrag, 1))
	Return pX + DRAGON(3, 1) + DRAGON(myDrag, 1)
End Function

; returns Y-coord of dragon, despite ownership
Function DragY#(myDrag)
	If DRAGON(myDrag, 0) > -1 Then Return DRAGON(myDrag, 2)
	If ((DRAGON(myDrag, 0) = -2) And (DRAGON(3, 0) <> -1)) Then Return Float(DRAGON(3, 2) + DRAGON(myDrag, 2))
	Return pY + DRAGON(3, 2) + DRAGON(myDrag, 2)
End Function

; tells what room an object's in, despite ownership
Function ObjRoom(myObj)
	If objPos(myObj, 0) > -1 Then Return objPos(myObj, 0)
	If objPos(myObj, 0) = -1 Then Return Room
	If ((objPos(myObj, 0) = -2) And (DRAGON(3, 0) <> -1)) Then Return DRAGON(3, 0)
	Return Room
End Function

; returns X-coord of an object, despite ownership
Function ObjX#(myObj)
	If objPos(myObj, 0) > -1 Then Return objPos(myObj, 1)
	If objPos(myObj, 0) = -1 Then Return Float(pX + objPos(myObj, 1))
	If ((objPos(myObj, 0) = -2) And (DRAGON(3, 0) <> -1)) Then Return Float(DRAGON(3, 1) + objPos(myObj, 1))
	Return pX + DRAGON(3, 1) + objPos(myObj, 1)
End Function

; returns Y-coord of an object, despite ownership
Function ObjY#(myObj)
	If objPos(myObj, 0) > -1 Then Return objPos(myObj, 2)
	If objPos(myObj, 0) = -1 Then Return Float(pY + objPos(myObj, 2))
	If ((objPos(myObj, 0) = -2) And (DRAGON(3, 0) <> -1)) Then Return Float(DRAGON(3, 2) + objPos(myObj, 2))
	Return pY + DRAGON(3, 2) + objPos(myObj, 2)
End Function


; drops whatever the bat's carrying
Function BatDropObj()
	If DRAGON(3, 4) = -1 Then Return
	
	If DRAGON(3, 4) > 99	; bat has a dragon
		myDrag = DRAGON(3, 4) - 100
		DRAGON(myDrag, 1) = DRAGON(3, 1) + DRAGON(myDrag, 1)
		DRAGON(myDrag, 2) = DRAGON(3, 2) + DRAGON(myDrag, 2)
		DRAGON(myDrag, 0) = DRAGON(3, 0)
		If DRAGON(myDrag, 3) = ZONE Then DRAGON(myDrag, 3) = HUNT
	Else
		ObjPos(DRAGON(3, 4), 1) = objX(DRAGON(3, 4))
		ObjPos(DRAGON(3, 4), 2) = objY(DRAGON(3, 4))
		ObjPos(DRAGON(3, 4), 0) = ObjRoom(DRAGON(3, 4))
	EndIf
	
	DRAGON(3, 4)= -1

End Function


; lets the bat get an object
Function BatGetObj(myObj)
	ObjPos(myObj, 0) = -2
	ObjPos(myObj, 1) = ((DRAGON(3, 1) - objpos(myObj, 1)) * -1.25) 
	ObjPos(myObj, 2) = ((DRAGON(3, 2) - objpos(myObj, 2)) * -1.25)
	DRAGON(3, 4) = myObj
	BatHappy = GameTime + Rand(5000, 30000)
	BatBored = False
End Function


Function BatGetDragon(drag)
	DRAGON(drag, 0) = -2
	DRAGON(drag, 1) = ((DRAGON(3, 1) - DRAGON(drag, 1)) * -1.25)
	DRAGON(drag, 2) = ((DRAGON(3, 2) - DRAGON(drag, 2)) * -1.25)
	If DRAGON(drag, 3) <> DEAD Then DRAGON(drag, 3) = ZONE
	DRAGON(3, 4) = 100 + drag
	BatHappy = GameTime + Rand(5000, 30000)
	BatBored = False
End Function


; drops whatever the player's carrying
Function DropObj()
	If Carry = -1 Then Return
	
	If Carry = 99	; carrying the bat!
		DRAGON(3, 0) = room
		DRAGON(3, 1) = px + DRAGON(3, 1)
		DRAGON(3, 2) = py + DRAGON(3, 2)
		DRAGON(3, 6) = Rand(1, 8)
	
	Else
		
		ObjPos(Carry, 1) = px + ObjPos(Carry, 1) 
		ObjPos(Carry, 2) = py + ObjPos(Carry, 2) 
		ObjPos(Carry, 0) = room		
	EndIf

	Carry = -1
	PlaySound SFX_DROP
End Function



; lets the player get the specified object
Function GetObj(obj)
	If (LastMove And WEST)
		VecX = -4
	ElseIf (lastMove And EAST)
		VecX = 4
	ElseIf (lastMove And NORTH)
		VecY = -4
	ElseIf (LastMove And SOUTH)
		VecY = 4
	EndIf
	
	ObjPos(obj, 0) = -1
	ObjPos(obj, 1) = ((px - objpos(obj, 1)) * -1) + vecX
	ObjPos(obj, 2) = ((py - objpos(obj, 2)) * -1) + vecY
	Carry = obj
	PlaySound SFX_PICK
End Function


; lets the player pick up the bat
Function GetBat()
	If (LastMove And WEST)
		VecX = -4
	ElseIf (lastMove And EAST)
		VecX = 4
	ElseIf (lastMove And NORTH)
		VecY = -4
	ElseIf (LastMove And SOUTH)
		VecY = 4
	EndIf
	
	DRAGON(3, 0) = -1
	DRAGON(3, 1) = ((px - DRAGON(3, 1)) * -1) + vecX
	DRAGON(3, 2) = ((py - DRAGON(3, 2)) * -1) + vecY
	Carry = 99	; special code for da bat
	BatHappy = GameTime + Rand(10000, 35000)	; he'll wait for a good time to try to escape...
	
	PlaySound SFX_PICK
End Function

; from here out it's just data statements... first is room connections, then default
; (game 1) item locations, then chalice glow info, then potential (game 2) object locations.

; obj0 = player
; obj1 = sword
; obj2 = magnet
; obj3 = bridge
; obj4 = gold key
; obj5 = black key
; obj6 = white key
; obj7 = chalice
; obj8 = microdot
; obj9 = bat (flap1)
; obj10 = bat (flap2)
; obj11 = yorgle normal
; obj12 = yorgle biting
; obj13 = yorgle dead
; obj14 = grundle normal
; obj15 = grundle biting
; obj16 = grundle dead
; obj17 = rhindle normal
; obj18 = rhindle biting
; obj19 = rhindle dead
; obj20+ = gates

; scr0 = gold castle vestibule
; scr1 = gold castle courtyard
; scr2 = gold crossroads
; scr3 = west of gold crossroads
; scr4 = east of gold crossroads
; scr5 = laby entrance
; scr6 = laby top
; scr7 = laby ship
; scr8 = laby nearing
; scr9 = laby closest
; scr10 = black castle courtyard
; scr11 = black castle vestibule
; scr12 = black castle catacombs entrance
; scr13 = black castle catacombs ne
; scr14 = black castle catacombs nw
; scr15 = black castle catacombs sw
; scr16 = catacombs entrance
; scr17 = catacombs mid
; scr18 = catacombs south
; scr19 = white crossroads
; scr20 = s of white crossroads
; scr21 = unused crossroads
; scr22 = n of unused crossroads
; scr23 = s of unused crossroads
; scr24 = white castle courtyard
; scr25 = white castle laby entrance
; scr26 = white castle laby ne
; scr27 = white castle laby nw
; scr28 = white castle laby sw

; gold vestibule (0)
Data 0, 0, 1, 0

; gold courtyard (1)
Data 3, 5, 2, 6

; gold cross (2)
Data 1, 4, 1, 3

; gold west (3)
Data 5, 2, 2, 4

; gold east (4)
Data 2, 3, 16, 2

; entrance (5)
Data 6, 8, 3, 8

; top (6)
Data 6, 7, 5, 9

; ship (7)
Data 8, 9, 6, 6

; nearing (8)
Data 9, 5, 7, 5

; closest (9)
Data 10, 6, 8, 7

; black court (10)
Data 7, 9, 9, 9

; black vestibule (11)
Data 12, 11, 10, 11

; black catacombs entrance (12)
Data 13, 14, 11, 15

; black catacombs ne (13)
Data 13, 15, 12, 14

; black catacombs nw (14)
Data 15, 13, 15, 12

; black catacombs se (15)
Data 14, 12, 14, 13

; main catacombs entrance (16)
Data 4, 17, 17, 17

; main catacombs mid (17)
Data 16, 16, 18, 16

; main catacombs south (18)
Data 17, 21, 16, 19

; white crossroads (19)
Data 24, 18, 20, 21

; white south (20)
Data 19, 23, 24, 22

; unused crossroads (21)
Data 22, 19, 23, 18

; unused north (22)
Data 23, 24, 21, 17

; unused  south (23)
Data 21, 20, 22, 18

; white court (24)
Data 20, 17, 19, 22

; white laby entrance (25)
Data 26, 28, 24, 28

; white laby ne (26)
Data 26, 27, 25, 27

; white laby nw (27)
Data 27, 26, 28, 26

; white laby sw (28)
Data 27, 25, 24, 25

; ITEMS
; =====
.items

; player
Data 0, 0, 0

; sword
Data 0, 50, 120

; magnet
Data 10, 50, 150

; bridge
Data 9, 75, 100

; gold key
Data 3, 50, 120

; black key
Data 28, 150, 150

; white key
Data 22, 50, 50

; chalice
Data 15, 120, 150
;Data 1, 50, 150	-- for win-condition testing purposes!

; microdot
Data 15, 1, 1


; CHALICE GLOW
; ============
.chalice

Data 136	; # of pixels
Data 0, 0
Data 1, 0
Data 14, 0
Data 15, 0
Data 0, 1
Data 1, 1
Data 14, 1
Data 15, 1
Data 0, 2
Data 1, 2
Data 14, 2
Data 15, 2
Data 0, 3
Data 1, 3
Data 14, 3
Data 15, 3
Data 0, 4
Data 1, 4
Data 2, 4
Data 3, 4
Data 12, 4
Data 13, 4
Data 14, 4
Data 15, 4
Data 0, 5
Data 1, 5
Data 2, 5
Data 3, 5
Data 12, 5
Data 13, 5
Data 14, 5
Data 15, 5
Data 2, 6
Data 3, 6
Data 4, 6
Data 5, 6
Data 6, 6
Data 7, 6
Data 8, 6
Data 9, 6
Data 10, 6
Data 11, 6
Data 12, 6
Data 13, 6
Data 2, 7
Data 3, 7
Data 4, 7
Data 5, 7
Data 6, 7
Data 7, 7
Data 8, 7
Data 9, 7
Data 10, 7
Data 11, 7
Data 12, 7
Data 13, 7
Data 2, 8
Data 3, 8
Data 4, 8
Data 5, 8
Data 6, 8
Data 7, 8
Data 8, 8
Data 9, 8
Data 10, 8
Data 11, 8
Data 12, 8
Data 13, 8
Data 2, 9
Data 3, 9
Data 4, 9
Data 5, 9
Data 6, 9
Data 7, 9
Data 8, 9
Data 9, 9
Data 10, 9
Data 11, 9
Data 12, 9
Data 13, 9
Data 4, 10
Data 5, 10
Data 6, 10
Data 7, 10
Data 8, 10
Data 9, 10
Data 10, 10
Data 11, 10
Data 4, 11
Data 5, 11
Data 6, 11
Data 7, 11
Data 8, 11
Data 9, 11
Data 10, 11
Data 11, 11
Data 6, 12
Data 7, 12
Data 8, 12
Data 9, 12
Data 6, 13
Data 7, 13
Data 8, 13
Data 9, 13
Data 6, 14
Data 7, 14
Data 8, 14
Data 9, 14
Data 6, 15
Data 7, 15
Data 8, 15
Data 9, 15
Data 2, 16
Data 3, 16
Data 4, 16
Data 5, 16
Data 6, 16
Data 7, 16
Data 8, 16
Data 9, 16
Data 10, 16
Data 11, 16
Data 12, 16
Data 13, 16
Data 2, 17
Data 3, 17
Data 4, 17
Data 5, 17
Data 6, 17
Data 7, 17
Data 8, 17
Data 9, 17
Data 10, 17
Data 11, 17
Data 12, 17
Data 13, 17


; data format: Screen, X, Y, Unsuitable, Chalice, weight (# entries)

; RANDOM OBJECT PLACEMENT
; =======================
.randPlace
Data 27
; gold vestibule
Data 0, 245, 115, 4, 0, 2		; gold key cannot go here!

; gold courtyard
Data 1, 60, 125, -1, 0, 3	

; gold crossroads west
Data 3, 80, 60, -1, 0, 1

; gold crossroads east
Data 4, 230, 60, -1, 0, 1

; blue labyrinth ship
Data 7, 25, 145, 3, 0, 1
Data 7, 275, 145, 3, 0, 1

; blue labyrinth crook
Data 8, 55, 80, 3, 0, 1
Data 8, 245, 80, 3, 0, 1

; blue labyrinth end
Data 9, 35, 90, -1, 0, 1
Data 9, 195, 145, 3, 0, 1

; black courtyard
Data 10, 60, 125, -1, 0, 3

; black vestibule
Data 11, 245, 115, 5, 1, 1

; black dungeon 1
Data 12, 65, 20, 5, 1, 1

; black dungeon 2
Data 13, 75, 85, 5, 1, 1
Data 13, 100, 145, 5, 1, 1

; black dungeon 3
Data 14, 115, 20, 5, 1, 1

; black dungeon 4
Data 15, 115, 145, 5, 1, 2
Data 15, 22, 145, 5, 1, 1

; catacombs 2
Data 17, 130, 145, -1, 0, 1

; catacombs 3
Data 18, 67, 145, -1, 0, 1

; white crossroads south
Data 20, 25, 120, -1, 0, 1

; unused crossroads north
Data 22, 25, 25, -1, 0, 1

; unused crossroads south
Data 23, 25, 120, -1, 0, 1

; white courtyard
Data 24, 60, 125, -1, 0, 3

; red laby 1
Data 25, 25, 83, 6, 0, 1		; no white key!

; red laby 2
Data 26, 86, 83, 6, 0, 1		; no white key!

; red laby 4
Data 28, 145, 140, 6, 1, 2		; no white key!